package com.tillekesoft.pinyin;

import net.sourceforge.pinyin4j.PinyinHelper;
import net.sourceforge.pinyin4j.format.HanyuPinyinCaseType;
import net.sourceforge.pinyin4j.format.HanyuPinyinOutputFormat;
import net.sourceforge.pinyin4j.format.HanyuPinyinToneType;
import net.sourceforge.pinyin4j.format.HanyuPinyinVCharType;
import net.sourceforge.pinyin4j.format.exception.BadHanyuPinyinOutputFormatCombination;

import java.util.ArrayList;
import java.util.Hashtable;
import java.util.List;
import java.util.Map;


public class Pinyin4jUtil {
    //int caseType, int toneType, int vCharType
    public static final int LOWERCASE = 1;
    public static final int UPPERCASE = 2;
    public static final int WITHOUT_TONE = 3;
    public static final int WITH_TONE_MARK = 4;
    public static final int WITH_TONE_NUMBER = 5;
    public static final int WITH_U_UNICODE = 6;
    public static final int WITH_U_AND_COLON = 7;
    public static final int WITH_V = 8;
    public static final int DO_NOT_USE_VCHARTYPE = 9;

    private static final String TAG = "B4A";

    public static String converterToFirstSpell(String chines) {
        StringBuilder pinyinName = new StringBuilder();
        char[] nameChar = chines.toCharArray();
        HanyuPinyinOutputFormat defaultFormat = new HanyuPinyinOutputFormat();
        defaultFormat.setCaseType(HanyuPinyinCaseType.LOWERCASE);
        defaultFormat.setToneType(HanyuPinyinToneType.WITHOUT_TONE);
        for (char aNameChar : nameChar) {
            if (aNameChar > 128) {
                try {
                    String[] strs = PinyinHelper.toHanyuPinyinStringArray(
                            aNameChar, defaultFormat);
                    if (strs != null) {
                        for (int j = 0; j < strs.length; j++) {
                            pinyinName.append(strs[j].charAt(0));
                            if (j != strs.length - 1) {
                                pinyinName.append(",");
                            }
                        }
                    }
                    // else {
                    // pinyinName.append(nameChar[i]);
                    // }
                } catch (BadHanyuPinyinOutputFormatCombination e) {
                    e.printStackTrace();
                }
            } else {
                pinyinName.append(aNameChar);
            }
            pinyinName.append(" ");
        }
        // return pinyinName.toString();
        return parseTheChineseByObject(discountTheChinese(pinyinName.toString()));
    }

    public static String converterToSpell(String chines) {
        StringBuilder pinyinName = new StringBuilder();
        char[] nameChar = chines.toCharArray();
        HanyuPinyinOutputFormat defaultFormat = new HanyuPinyinOutputFormat();
        defaultFormat.setCaseType(HanyuPinyinCaseType.LOWERCASE);
        defaultFormat.setToneType(HanyuPinyinToneType.WITHOUT_TONE);
        for (char aNameChar : nameChar) {
            if (aNameChar > 128) {
                try {
                    String[] strs = PinyinHelper.toHanyuPinyinStringArray(
                            aNameChar, defaultFormat);
                    if (strs != null) {
                        for (int j = 0; j < strs.length; j++) {
                            pinyinName.append(strs[j]);
                            if (j != strs.length - 1) {
                                pinyinName.append(",");
                            }
                        }
                    }
                } catch (BadHanyuPinyinOutputFormatCombination e) {
                    e.printStackTrace();
                }
            } else {
                pinyinName.append(aNameChar);
            }
            pinyinName.append(" ");
        }
        // return pinyinName.toString();
        return parseTheChineseByObject(discountTheChinese(pinyinName.toString()));
    }




    public static String converterToSpell2(String chines, int caseType, int toneType, int vCharType ) {
        StringBuilder pinyinName = new StringBuilder();
        char[] nameChar = chines.toCharArray();
        HanyuPinyinOutputFormat defaultFormat = new HanyuPinyinOutputFormat();
//        Log.d(TAG, "values of parameters are " + caseType + " " + toneType + " " + vCharType);
        if (caseType == LOWERCASE) {
//            Log.d(TAG, "caseType IF = LOWERCASE");
            defaultFormat.setCaseType(HanyuPinyinCaseType.LOWERCASE);
        } else if (caseType == UPPERCASE ) {
//            Log.d(TAG, "caseType IF = UPPERCASE");
            defaultFormat.setCaseType(HanyuPinyinCaseType.UPPERCASE);
        }

        if (toneType == WITHOUT_TONE) {
//            Log.d(TAG, "toneType IF = WITHOUT_TONE");
            defaultFormat.setToneType(HanyuPinyinToneType.WITHOUT_TONE);
        } else if (toneType == WITH_TONE_MARK){
//            Log.d(TAG, "toneType IF = WITH_TONE_MARK");
            defaultFormat.setToneType(HanyuPinyinToneType.WITH_TONE_MARK);
        } else if (toneType == WITH_TONE_NUMBER) {
//            Log.d(TAG, "toneType IF = WITH_TONE_NUMBER");
            defaultFormat.setToneType(HanyuPinyinToneType.WITH_TONE_NUMBER);
        }


        if (vCharType != DO_NOT_USE_VCHARTYPE) {
            if (vCharType == WITH_U_UNICODE) {
//                Log.d(TAG, "vCharType IF = WITH_U_UNICODE");
                defaultFormat.setVCharType(HanyuPinyinVCharType.WITH_U_UNICODE);
            } else if (vCharType == WITH_U_AND_COLON) {
//                Log.d(TAG, "vCharType IF = WITH_U_AND_COLON");
                defaultFormat.setVCharType(HanyuPinyinVCharType.WITH_U_AND_COLON);
            } else if (vCharType == WITH_V) {
//                Log.d(TAG, "vCharType IF = WITH_V");
                defaultFormat.setVCharType(HanyuPinyinVCharType.WITH_V);
            }
        }


        for (char aNameChar : nameChar) {
            if (aNameChar > 128) {
                try {
                    String[] strs = PinyinHelper.toHanyuPinyinStringArray(
                            aNameChar, defaultFormat);
                    if (strs != null) {
                        for (int j = 0; j < strs.length; j++) {
                            pinyinName.append(strs[j]);
                            if (j != strs.length - 1) {
                                pinyinName.append(",");
                            }
                        }
                    }
                } catch (BadHanyuPinyinOutputFormatCombination e) {
                    e.printStackTrace();
                }
            } else {
                pinyinName.append(aNameChar);
            }
            pinyinName.append(" ");
        }
        // return pinyinName.toString();
        return parseTheChineseByObject(discountTheChinese(pinyinName.toString()));
    }


    public static String HanZiToPinYinWIthToneMark(char hanzi){
        HanyuPinyinOutputFormat hanyuPinyin = new HanyuPinyinOutputFormat();
        hanyuPinyin.setCaseType(HanyuPinyinCaseType.LOWERCASE);
        hanyuPinyin.setToneType(HanyuPinyinToneType.WITH_TONE_MARK);
        hanyuPinyin.setVCharType(HanyuPinyinVCharType.WITH_U_UNICODE);
        String[] pinyinArray=null;
        try {
            if(hanzi>=0x4e00 && hanzi<=0x9fa5){
                pinyinArray = PinyinHelper.toHanyuPinyinStringArray(hanzi, hanyuPinyin);
            }
        } catch (BadHanyuPinyinOutputFormatCombination e) {
            e.printStackTrace();
        }
        return pinyinArray[0];
    }


    private static List<Map<String, Integer>> discountTheChinese(String theStr) {
        List<Map<String, Integer>> mapList = new ArrayList<>();
        Map<String, Integer> onlyOne;
        String[] firsts = theStr.split(" ");
        for (String str : firsts) {
            onlyOne = new Hashtable<>();
            String[] china = str.split(",");
            for (String s : china) {
                Integer count = onlyOne.get(s);
                if (count == null) {
                    onlyOne.put(s, 1);
                } else {
                    onlyOne.remove(s);
                    count++;
                    onlyOne.put(s, count);
                }
            }
            mapList.add(onlyOne);
        }
        return mapList;
    }

    private static String parseTheChineseByObject(
            List<Map<String, Integer>> list) {
        Map<String, Integer> first = null;
        for (int i = 0; i < list.size(); i++) {
            Map<String, Integer> temp = new Hashtable<>();
            if (first != null) {
                for (String s : first.keySet()) {
                    for (String s1 : list.get(i).keySet()) {
                        String str = s + s1;
                        temp.put(str, 1);
                    }
                }
                if (temp.size() > 0) {
                    first.clear();
                }
            } else {
                for (String s : list.get(i).keySet()) {
                    temp.put(s, 1);
                }
            }
            if (temp.size() > 0) {
                first = temp;
            }
        }
        String returnStr = "";
        if (first != null) {
            for (String str : first.keySet()) {
                returnStr += (str + ",");
            }
        }
        if (returnStr.length() > 0) {
            returnStr = returnStr.substring(0, returnStr.length() - 1);
        }
        return returnStr;
    }

}